import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';

import { MapDrawingEntity } from '../draw/coordinate.entity';
import { MapDrawingDTO, ReturnDrawingsDTO } from './mapdrawing.dto';
import { NotificationGateway } from 'src/notifications/notifications.gateway';

@Injectable()
export class DrawService {
  constructor(
    @InjectRepository(MapDrawingEntity)
    private mapDrawingRepository: Repository<MapDrawingEntity>,
    private notificationGateway: NotificationGateway,
  ) {}

  async draw(gameId, data: MapDrawingDTO) {
    data['gameId'] = gameId;
    const drawing = await this.mapDrawingRepository.create(data);
    this.notificationGateway.server.emit(gameId, {
      type: 'drawing-update',
    });
    if (data.mapDrawingId == null || data.mapDrawingId == '') {
      // luo uuden instanssin.
      const mapDrawing = await this.mapDrawingRepository.insert(drawing);
      return mapDrawing.identifiers;
    } else {
      //päivittää mapDrawingin
      return await this.mapDrawingRepository.save(drawing);
    }
  }

  // draw map based on game and
  async drawMap(id, data: ReturnDrawingsDTO) {
    // return mapdrawings with given faction and gameid
    return await this.mapDrawingRepository.find({
      where: [
        {
          gameId: id,
          faction: data.factionId,
          drawingIsActive: true,
        },
        {
          gameId: id,
          faction: null,
          drawingIsActive: true,
        },
      ],
    });
  }
}
