import { Controller, Post, UsePipes, Body, Param, Get } from '@nestjs/common';

import { ValidationPipe } from '../shared/validation.pipe';
import { ScoreService } from './score.service';
import { ScoreDTO } from './score.dto';
import { GameEntity } from '../game/game.entity';
import { Roles, GameStates } from '../shared/guard.decorator';

@Controller('score')
export class ScoreController {
  constructor(private scoreService: ScoreService) {}

  // adds score manually to Faction
  // :id is gameId
  @Post('add-score/:id')
  @Roles('admin')
  @GameStates('STARTED')
  @UsePipes(new ValidationPipe())
  async addingScore(@Body() data: ScoreDTO, @Param('id') gameId: GameEntity) {
    return this.scoreService.addScore(data, gameId);
  }

  // temporary scoreTick path, :id is gameId
  @Get('tick-score/:id')
  @GameStates('STARTED')
  async scoreTick(@Param('id') gameId: GameEntity) {
    return this.scoreService.scoreTick(gameId);
  }

  // shows scores, :id is gameId 
  @Get('get-score/:id')
  @GameStates('STARTED')
  async getScores(@Param('id') gameId: GameEntity) {
    return this.scoreService.getScores(gameId);
  }
}
