import { Injectable, HttpStatus, HttpException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';

import { Game_PersonEntity } from '../game/game.entity';
import { TrackingEntity } from './tracking.entity';
import { TrackingDTO } from './tracking.dto';
import { FactionEntity } from '../faction/faction.entity';
import { NotificationGateway } from 'src/notifications/notifications.gateway';
import { GeoDTO } from './geo.dto';

@Injectable()
export class TrackingService {
  constructor(
    @InjectRepository(TrackingEntity)
    private trackingrepository: Repository<TrackingEntity>,
    @InjectRepository(Game_PersonEntity)
    private gamepersonrepository: Repository<Game_PersonEntity>,
  ) {}

  async trackLocation(personId, gameId, trackdata: GeoDTO) {
    // find player
    let gameperson = await this.gamepersonrepository.findOne({
      where: { game: gameId, person: personId },
      relations: ['faction'],
    });
    if (!gameperson) {
      throw new HttpException(
        'You have not joined this game',
        HttpStatus.BAD_REQUEST,
      );
    }
    let trackedperson = await this.trackingrepository.findOne({
      gamepersonId: gameperson,
    });

    // if player has pushed tracking data, update entry
    if (trackedperson) {
      trackdata['time'] = Date.now();
      //add coordinates
      trackedperson.data.push(trackdata);
      //add timestamp
      await this.trackingrepository.save(trackedperson);
      return { code: 201, message: 'Location updated!' };
    } else {
      // first entry will be empty
      trackdata['time'] = Date.now();
      // initialize data
      trackedperson = await this.trackingrepository.create(trackedperson);
      trackedperson.data = [trackdata];
      trackedperson.faction = gameperson.faction;
      trackedperson.game = gameId;
      trackedperson.gamepersonId = gameperson;
      await this.trackingrepository.save(trackedperson);

      return { code: 201, message: 'Entry Created!' };
    }
  }

  // get player data while game is running
  async getPlayers(userId, gameId) {
    // get gameperson
    const gameperson = await this.gamepersonrepository.findOne({
      where: { person: userId, game: gameId },
      relations: ['faction'],
    });

    let playerdata;
    // get playerdata
    if (gameperson.faction) {
      playerdata = await this.trackingrepository.find({
        where: { faction: gameperson.faction },
        relations: ['faction', 'gamepersonId'],
      });
    } else {
      playerdata = await this.trackingrepository.find({
        where: { game: gameId },
        relations: ['faction', 'gamepersonId'],
      });
    }

    // parse data
    const currentdata = await Promise.all(
      playerdata.map(async player => {
        return {
          gamepersonId: player['gamepersonId']['gamepersonId'],
          gamepersonRole: player['gamepersonId']['role'],
          factionId: player['faction']['factionId'],
          coordinates: player['data'].pop(),
        };
      }),
    );

    return currentdata;
  }

  private async mapFunction(data): Promise<Number> {
    return await data.map(type => {
      return type;
    });
  }
}
